@@ -1,13 +1,12 @@
 (**************************************************************************)
 (*    Lablgtk - Examples                                                  *)
 (*                                                                        *)
-(*    There is no specific licensing policy, but you may freely           *)
-(*    take inspiration from the code, and copy parts of it in your        *)
-(*    application.                                                        *)
+(*    This code is in the public domain.                                  *)
+(*    You may freely copy parts of it in your application.                *)
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id: testgtk.ml 1419 2008-09-22 13:37:06Z zoggy $ *)
+(* $Id$ *)
 
 open StdLabels
 open GdkKeysyms
@@ -86,7 +85,7 @@ let create_buttons =
 	    ~row_spacings:3 ~col_spacings:3 ~border_width:10
 	    ~packing:box1#add () in
 
-	let button = Array.create 9 (GButton.button ~label:"button1" ()) in
+	let button = Array.make 9 (GButton.button ~label:"button1" ()) in
 	for i = 2 to 9 do
 	  button.(i-1) <- GButton.button ~label:("button" ^ string_of_int i) ();
 	done;
@@ -448,8 +447,8 @@ let create_scrolled_windows =
 
 let make_toolbar (toolbar : GButton.toolbar) window =
   let icon =
-    let info = GDraw.pixmap_from_xpm ~file:"test.xpm" ~window () in
-    fun () -> (GMisc.pixmap info ())#coerce
+    let pb = GdkPixbuf.from_file "test.xpm" in
+    fun () -> (GMisc.image ~pixbuf:pb ())#coerce
   in
 
   toolbar #insert_button ~text:"Horizontal"
