@@ -20,7 +20,7 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* $Id: lablgladecc.ml 1359 2007-08-17 02:38:53Z garrigue $ *)
+(* $Id$ *)
 
 open StdLabels
 open Printf
@@ -170,6 +170,7 @@ type wtree = {
     wclass: string;
     wname: string;
     wcamlname : string;
+    winternal: bool;
     wchildren: wtree list;
     mutable wrapped: bool;
   }
@@ -180,13 +181,14 @@ exception Unsupported
 
 let camlize s = match s with 
   | "" -> "_"
-  |  s -> let s = String.uncapitalize s in
-     for i = 0 to String.length s - 1 do 
-       match s.[i] with
+  |  s -> let s = String.uncapitalize_ascii s in
+     let s = Bytes.unsafe_of_string s in
+     for i = 0 to Bytes.length s - 1 do 
+       match Bytes.get s i with
        | 'a'..'z'| 'A'..'Z' | '0'..'9' -> ()
-       | _ -> s.[i] <- '_'
+       | _ -> Bytes.set s i '_'
      done;
-     s
+     Bytes.unsafe_to_string s
 
 (* this name is a default one created by glade? *)
 let is_default_name s =
@@ -206,18 +208,22 @@ let is_default_name s =
 
 let is_top_widget wtree w =
   match wtree.wchildren with
-  | [w'] -> w.wcamlname = w'.wcamlname
+  | [w'] -> w.wcamlname = w'.wcamlname && not w.winternal
   | _ -> false
 
-let rec parse_widget ~wclass ~wname lexbuf =
+let rec parse_widget ~wclass ~wname ~internal lexbuf =
   let widgets = ref [] in
   while match token lexbuf with
   | Tag ("widget", attrs, closed) ->
-      widgets := parse_widget ~wclass:(List.assoc "class" attrs)
+      widgets := parse_widget ~wclass:(List.assoc "class" attrs) ~internal
 	  ~wname:(List.assoc "id" attrs) lexbuf :: !widgets;
       true
-  | Tag ("child",_,_) | Endtag "child" ->
+  | Tag ("child",attrs,_) ->
+      let is_internal =
+	try List.assoc "internal-child" attrs <> "" with Not_found -> false in
+      Stack.push is_internal internal;
       true
+  | Endtag "child" -> ignore(Stack.pop internal); true
   | Tag (tag,_,closed) ->
       if not closed then while token lexbuf <> Endtag tag do () done; true
   | Endtag "widget" ->
@@ -227,8 +233,9 @@ let rec parse_widget ~wclass ~wname lexbuf =
   | Endtag _ | EOF ->
       failwith "bad XML syntax"
   do () done;
+  let internal = try Stack.top internal with _ -> false in
   { wclass = wclass; wname = wname; wcamlname = camlize wname;
-    wchildren = List.rev !widgets; wrapped = false }
+    winternal = internal; wchildren = List.rev !widgets; wrapped = false }
 
 let rec flatten_tree w =
   let children = List.map ~f:flatten_tree w.wchildren in
@@ -290,7 +297,8 @@ let output_wrapper ~file wtree =
   begin match wtree.wchildren with
   | [w] ->
       printf "    method reparent parent =\n";
-      printf "      %s#misc#reparent parent;\n" w.wcamlname;
+      if not (is_hidden w) then
+          printf "      %s#misc#reparent parent;\n" w.wcamlname;
       printf "      toplevel#destroy ()\n";
   | _ -> ()
   end;
@@ -322,6 +330,7 @@ let parse_body ~file lexbuf =
   | Tag("widget", attrs, false) ->
       let wtree = 
 	parse_widget ~wclass:(List.assoc "class" attrs)
+	  ~internal:(Stack.create ())
 	  ~wname:(List.assoc "id" attrs) lexbuf 
       in
       let rec output_roots wtree =
@@ -346,7 +355,7 @@ let process ?(file="<stdin>") chan =
       let buf = String.create 1024 in
       while
         let len = input chan buf 0 1024 in
-        Buffer.add_substring b buf 0 len;
+        Buffer.add_subbytes b buf 0 len;
         len > 0
       do () done;
       let data = Buffer.contents b in
@@ -371,7 +380,7 @@ let output_test () =
   List.iter !classes ~f:
     begin fun (clas, _) ->
       output_widget
-        {wname = "a"^clas; wcamlname = camlize ("a"^clas);
+        {wname = "a"^clas; wcamlname = camlize ("a"^clas); winternal=false;
 	 wclass = clas; wchildren = []; wrapped = true}
     end;
   print_string "  end\n\n";
