@@ -29,6 +29,7 @@
 #include <caml/memory.h>
 #include <caml/callback.h>
 #include <caml/fail.h>
+#include <caml/threads.h>
 
 #include "wrappers.h"
 #include "ml_glib.h"
@@ -178,7 +179,7 @@ static void notify_destroy(gpointer unit, GClosure *c)
     remove_global_root((value*)&c->data);
 }
 
-static void marshal (GClosure *closure, GValue *ret,
+static void marshal_core (GClosure *closure, GValue *ret,
                      guint nargs, const GValue *args,
                      gpointer hint, gpointer marshall_data)
 {
@@ -194,6 +195,20 @@ static void marshal (GClosure *closure, GValue *ret,
     CAMLreturn0;
 }
 
+static void marshal (GClosure *closure, GValue *ret,
+                     guint nargs, const GValue *args,
+                     gpointer hint, gpointer marshall_data)
+{
+    if (polling) { // https://github.com/garrigue/lablgtk/issues/141
+      caml_leave_blocking_section();
+      polling = 0;
+      marshal_core(closure, ret, nargs, args, hint, marshall_data);
+      polling = 1;
+      caml_enter_blocking_section();
+    } else
+      marshal_core(closure, ret, nargs, args, hint, marshall_data);
+}
+
 CAMLprim value ml_g_closure_new (value clos)
 {
     GClosure* closure = g_closure_new_simple(sizeof(GClosure), (gpointer)clos);
@@ -643,8 +658,8 @@ ML_2 (g_signal_handler_is_connected, GObject_val, Long_val, Val_bool)
 ML_2 (g_signal_stop_emission_by_name, GObject_val, String_val, Unit)
 CAMLprim value ml_g_signal_emit_by_name (value obj, value sig, value params)
 {
-    CAMLparam3(obj,sig,params);
-    CAMLlocal1(ret);
+    value ret = Val_unit;
+    CAMLparam4(obj,sig,params,ret);
     GObject *instance = GObject_val(obj);
     GValue *iparams = (GValue*)calloc(1 + Wosize_val(params), sizeof(GValue));
     GQuark detail = 0;
@@ -671,11 +686,11 @@ CAMLprim value ml_g_signal_emit_by_name (value obj, value sig, value params)
                       query.param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE);
         g_value_set_mlvariant (&iparams[i+1], Field(params,i));
     }
-    g_signal_emitv (iparams, signal_id, detail, (ret ? GValue_val(ret) : 0));
+    g_signal_emitv (iparams, signal_id, detail,
+                    (ret == Val_unit ? 0 : GValue_val(ret)));
     for (i = 0; i < query.n_params + 1; i++)
         g_value_unset (iparams + i);
     free (iparams);
-    if (!ret) ret = Val_unit;
     CAMLreturn(ret);
 }
 
