@@ -33,16 +33,25 @@
 #include <caml/mlvalues.h>
 #include <caml/fail.h>
 #include <caml/custom.h>
-CAMLextern char *young_start, *young_end; /* from minor_gc.h */
+#include <caml/minor_gc.h>
+#define Is_young_block(v) \
+  (Is_block(v) && (value*)(v) < caml_young_end && (value*)(v) > caml_young_start)
+
+#ifndef Bytes_val
+#define Bytes_val String_val
+#endif
 
 CAMLexport value copy_memblock_indirected (void *src, asize_t size);
 value alloc_memblock_indirected (asize_t size);
+CAMLexport value ml_alloc_custom(struct custom_operations * ops,
+                                 uintnat size, mlsize_t mem, mlsize_t max);
 CAMLprim value ml_some (value);
 value ml_cons (value, value);
 CAMLexport void ml_raise_null_pointer (void) Noreturn;
 CAMLexport value Val_pointer (void *);
 CAMLprim value copy_string_check (const char*);
 value copy_string_or_null (const char *);
+value Val_option_string (const char *s);
 
 value string_list_of_strv (const char * const *v);
 value string_list_of_strv2 (char **v);
@@ -51,8 +60,19 @@ char ** strv_of_string_list (value list);
 CAMLprim value *ml_global_root_new (value v);
 CAMLexport void ml_global_root_destroy (void *data);
 
+/* Pointer conversions */
+#define Pointer_val(val) ((void*)Field(val,1))
+#define Store_pointer(val,p) (Field(val,1)=Val_bp(p))
+#define MLPointer_val(val) \
+        ((int)Field(val,1) == 2 ? &Field(val,2) : (void*)Field(val,1))
+
+#define Val_addr(ptr) (1+(value)ptr)
+#define Addr_val(val) ((void*)(val-1))
+
 /* enums <-> polymorphic variants */
 typedef struct { value key; int data; } lookup_info;
+#define Val_lookup_info(v) Val_pointer((void*)v)
+#define Lookup_info_val(v) ((const lookup_info*)Pointer_val(v))
 CAMLexport value ml_lookup_from_c (const lookup_info table[], int data);
 CAMLexport int ml_lookup_to_c (const lookup_info table[], value key);
 CAMLexport value ml_lookup_flags_getter (const lookup_info table[], int data);
@@ -254,7 +274,8 @@ CAMLprim value cname##_bc (value *argv, int argn) \
 /* parameter conversion */
 #define Bool_ptr(x) ((long) x - 1)
 #define Char_val Int_val
-#define Float_val(x) ((float)Double_val(x))
+#define Float_val Double_val
+/* #define Float_val(x) ((float)Double_val(x)) */
 #define SizedString_val(x) String_val(x), string_length(x)
 
 #define Option_val(val,unwrap,default) \
@@ -276,7 +297,7 @@ CAMLprim value cname##_bc (value *argv, int argn) \
  else if (l <= Max_young_wosize) { int i; ret = alloc_tuple(l); \
    for(i=0;i<l;i++) Field(ret,i) = conv(src[i]); } \
  else { int i; ret = alloc_shr(l,0); \
-   for(i=0;i<l;i++) initialize (&Field(ret,i), conv(src[i])); }
+   for(i=0;i<l;i++) caml_initialize (&Field(ret,i), conv(src[i])); }
 
 #define Make_Val_final_pointer(type, init, final, adv) \
 static void ml_final_##type (value val) \
@@ -286,8 +307,8 @@ static struct custom_operations ml_custom_##type = \
   custom_hash_default, custom_serialize_default, custom_deserialize_default };\
 CAMLprim value Val_##type (type *p) \
 { value ret; if (!p) ml_raise_null_pointer(); \
-  ret = alloc_custom (&ml_custom_##type, sizeof(value), adv, 1000); \
-  initialize (&Field(ret,1), (value) p); init(p); return ret; }
+  ret = ml_alloc_custom (&ml_custom_##type, sizeof(value), adv, 1000); \
+  caml_initialize (&Field(ret,1), (value) p); init(p); return ret; }
 
 #define Make_Val_final_pointer_ext(type, ext, init, final, adv) \
 static void ml_final_##type##ext (value val) \
@@ -297,8 +318,8 @@ static struct custom_operations ml_custom_##type##ext = \
   custom_hash_default, custom_serialize_default, custom_deserialize_default };\
 CAMLprim value Val_##type##ext (type *p) \
 { value ret; if (!p) ml_raise_null_pointer(); \
-  ret = alloc_custom (&ml_custom_##type##ext, sizeof(value), adv, 1000); \
-  initialize (&Field(ret,1), (value) p); init(p); return ret; }
+  ret = ml_alloc_custom (&ml_custom_##type##ext, sizeof(value), adv, 1000); \
+  caml_initialize (&Field(ret,1), (value) p); init(p); return ret; }
 
 #define Make_Val_final_pointer_compare(type, init, comp, final, adv) \
 static void ml_final_##type (value val) \
@@ -310,16 +331,8 @@ static struct custom_operations ml_custom_##type = \
   custom_hash_default, custom_serialize_default, custom_deserialize_default };\
 CAMLprim value Val_##type (type *p) \
 { value ret; if (!p) ml_raise_null_pointer(); \
-  ret = alloc_custom (&ml_custom_##type, sizeof(value), adv, 1000); \
-  initialize (&Field(ret,1), (value) p); init(p); return ret; }
-
-#define Pointer_val(val) ((void*)Field(val,1))
-#define Store_pointer(val,p) (Field(val,1)=Val_bp(p))
-#define MLPointer_val(val) \
-        ((int)Field(val,1) == 2 ? &Field(val,2) : (void*)Field(val,1))
-
-#define Val_addr(ptr) (1+(value)ptr)
-#define Addr_val(val) ((void*)(val-1))
+  ret = ml_alloc_custom (&ml_custom_##type, sizeof(value), adv, 1000); \
+  caml_initialize (&Field(ret,1), (value) p); init(p); return ret; }
 
 #define Wosize_asize(x) ((x-1)/sizeof(value)+1)
 #define Wosizeof(x) Wosize_asize(sizeof(x))
